#!/usr/local/bin/python
###!/usr/bin/env python3.1

import os
import sys
import stat
import glob
import random
import tarfile
import shutil



if len(sys.argv) != 2:
  sys.exit( 'USAGE : ' + sys.argv[0] + ' <end_of_job_file.tgz>\n' )

# Keep track of the Current Working Directory
my_cwd = os.getcwd()

# Enter the experiment base dir (if not already inside)
if sys.argv[1].find('/') != -1:
  part_string           = sys.argv[1].rpartition('/')
  exp_base_path         = part_string[0]
  end_of_job_file_name  = part_string[2]
  os.chdir( exp_base_path )
else:
 end_of_job_file_name   = sys.argv[1]
 
# Untar end_of_job_file
end_of_job_file = tarfile.open( end_of_job_file_name, 'r:gz')
end_of_job_file.extractall()
end_of_job_file.close()

# Reconstruct sim_dir
part_string   = end_of_job_file_name.rpartition( '_seed' )
part_string2  = part_string[2].rpartition( '.' )
sim_dir       = part_string[0] + '/seed' + part_string2[0]

script_dir = 'scripts'


# Get rid of the ending '/' in sim_dir and script_dir (if any)
sim_dir     = sim_dir.rstrip( '/' )
script_dir  = script_dir.rstrip( '/' )

# Get exp name
exp_name_file = open( sim_dir + '/exp_name.txt', 'r' )
exp_name = (exp_name_file.readline()).rstrip( '\n' )
exp_name_file.close()

# Get simulation name
sim_name_file = open( sim_dir + '/sim_name.txt', 'r' )
sim_name = (sim_name_file.readline()).rstrip( '\n' )
sim_name_file.close()

# Get number of the last saved generation
last_gener_file = open( sim_dir + '/last_gener.txt', 'r' )
last_gener = int( (last_gener_file.readline()).rstrip( '\n' ) )
last_gener_file.close()

# Get number of generations to do 
nb_gener_file = open( 'nb_gener.txt', 'r' )
nb_gener = int( (nb_gener_file.readline()).rstrip( '\n' ) )
nb_gener_file.close()


if last_gener >= nb_gener:
  sys.exit( sys.argv[1] + ': Simulation is finished\n' )



# **********************************************************************
# Create the "out_files.tgz" archive
# **********************************************************************
os.chdir( sim_dir )
out_files = tarfile.open( 'out_files.tgz', 'w:gz')
for filename in glob.glob( os.path.join( '.', '*.out' ) ):
  out_files.add( filename )
out_files.close()
os.chdir( '../..' )


# **********************************************************************
# Create the new script file
# **********************************************************************
os.chdir( script_dir )
old_script_file_name = sim_name + '.sh'
# NB : '{0:0>6}'.format( foo ) forces foo to span 6 digits (filled with 0s on the left)
new_script_file_name = sim_name + '_' + '{0:0>6}'.format( last_gener ) + '.sh'

old_script_file = open( old_script_file_name, 'r' )
new_script_file = open( new_script_file_name, 'w' )

# Copy the content of old_script_file into new_script_file with a few modifications : 
#   * Instead of copying the param.in file, we need to retreive the last backup and all the
#     .out files (mainly the stat files)
#   * The simulation must be run for <nb_gener> - <last_gener> simulation starting from 
#     generation <last_gener>
for cur_line in old_script_file:
  if cur_line.find( '# Copy the param.in file' ) == 0:
    new_script_file.write( '# Retreive backed files necessary to continue the simulation\n' )
  elif cur_line.find( 'cp $SPS_SIM_DIR"/param.in"' ) == 0:
    new_script_file.write( 'mkdir backup tree\n' )
    new_script_file.write( 'SPS_LAST_BACKUP=$SPS_SIM_DIR"/backup/gen_' + '{0:0>6}'.format( last_gener ) + '.ae"\n' )
    new_script_file.write( 'cp $SPS_LAST_BACKUP "backup/"\n' )
    new_script_file.write( 'cp $SPS_SIM_DIR"/out_files.tgz" .\n' )
    new_script_file.write( 'tar zxf out_files.tgz\n' )
    new_script_file.write( 'rm out_files.tgz\n' )
  elif cur_line.find( '$EXEC' ) == 0:
    new_script_file.write( '$EXEC -f backup/gen_' + '{0:0>6}'.format( last_gener ) + '.ae -n ' + str(nb_gener-last_gener) + ' > /dev/null\n' )
  else:
    new_script_file.write( cur_line )

old_script_file.close()
new_script_file.close()
os.chmod(new_script_file_name, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH) # chmod 744

# **********************************************************************
#                           SUBMIT NEW JOB
# **********************************************************************
os.system( 'qsub ' + new_script_file_name )


# Go back to the directory we were in at the beginning
os.chdir( my_cwd )



#!/usr/local/bin/python

import os
import sys
import glob

for simu in glob.glob( os.path.join( '.', '*.tgz' ) ):
  simu_name = simu.rpartition( '.tgz' )[0]
  continue_script = sys.argv[0].rpartition( '/' )[0] + '/continue_from_last_backup.py'
  
  print( 'Preparing ' + simu_name + '...\n' )
  os.system( continue_script + ' ' + simu )
  os.system( 'mv '+ simu_name+'.tgz ' + simu_name+'.tgz.bak' )




#!/usr/bin/env python
import os
from pprint import pprint
import sys
import getopt
import numpy as np
import csv
import linecache

def main(argv):
    result_dir="defaultdir"
    output_csv="default.csv"
    try:
        opts, extraparams = getopt.getopt(sys.argv[1:],"hi:o:",["input-dir=","output-csv="]) 
    except getopt.GetoptError:
        print 'test.py -i <inputdirectory> -o <outputfile.csv>'
        sys.exit(2)

    for o,p in opts:
        if o == '-h':
            print 'execoToCSV.py -i <inputdir> -o <outputcsv>'
            print 'inputdir: a input directory where the EXECO results are stored'
            print 'outputdir: an output csv file where the results will be stored'
            sys.exit(2)
        elif o in ['-i','--input-dir']:
            result_dir = p
        elif o in ['-o','--output-csv']:
            output_csv = p


    if result_dir == "defaultdir":
        print "You must define a input directory where the EXECO results are stored"
        sys.exit()

    if output_csv == "default.csv":
        print "You must define an output csv file where the results will be stored"
        sys.exit()

    print 'Input result directory:',result_dir
    print 'Output CSV file: ',output_csv

    list_comb = os.listdir(result_dir)
   
    with open(output_csv, 'wb') as csvfile:
      csvwriter = csv.writer(csvfile)
      with open('toredo_xp.csv','wb') as csvfnotfinish:
	fnotfinish = csv.writer(csvfnotfinish)
	
	for comb_dir in list_comb:
	  print comb_dir
	  k = comb_dir.replace('/',' ').split('-')
	  i = iter(k)
	  params = dict(zip(i,i))
	  
	  if os.path.isfile(result_dir+'/'+comb_dir+"/logger_csv.log"):
	    with open(result_dir+'/'+comb_dir+"/logger_csv.log") as f:
		for line in f:
		  if line.split(",")[0] == 'SELECTION' and len(line.split(",")) == 3:
		    csvwriter.writerow([params['seed'],params['experiment'],params['fuzzy'],params['compilator'],
			  params['parallel'],params['number_of_generation'],int(line.split(",")[1]),int(line.split(",")[2])])
	    
	    
if __name__ == "__main__":
   main(sys.argv[1:])


#!/usr/bin/env python
# encoding=utf8

import os, time, datetime
from tempfile import mkstemp
import copy
import hashlib
import shutil

from threading import Thread
from execo import Put, Remote, Get, sleep, default_connection_params, Host, format_date, format_duration, SshProcess, Process
from execo.log import style
from execo import logger as ex_log
from execo.time_utils import timedelta_to_seconds
from execo_g5k import get_host_attributes, get_planning, compute_slots, \
    find_first_slot, distribute_hosts, get_jobs_specs, g5k_configuration, \
    wait_oargrid_job_start, oargridsub, oargriddel, get_oargrid_job_nodes, \
    Deployment, deploy, get_oargrid_job_info, get_host_cluster, get_cluster_site, get_host_site, get_g5k_sites, get_oar_job_info, \
    get_cluster_site, OarSubmission, \
    oarsub, get_oar_job_nodes, wait_oar_job_start, oardel, get_host_attributes
from execo_engine import Engine, logger, ParamSweeper, sweep, slugify


class compil_aevol(Engine):

    def __init__(self):
        """Overloading class initialization with parent and adding options"""
        super(compil_aevol, self).__init__()

    def run(self):
        """ """
        try:
            # Creation of the main iterator which is used for the first control loop.
            self.define_parameters()
	    
            # While there are combinations to treat
            while len(self.sweeper.get_remaining()) > 0:
	      comb = self.sweeper.get_next()
              if comb:
		self.workflow(comb)
	finally:
	  logger.info("Compilation DONE")
	  
    def define_parameters(self):
        """ """
        parameters = {
          'seed' : [51456165, 33263658, 7158785, 456847894, 1223144, 878944, 121145, 3587842, 2784564, 68984554],
          'mutation' : ['5e-5','1e-5','5e-6','1e-6','5e-7'],
          'env' : ['const','lat_3','lat_13','lat_123','lat_all'],
          'selection' : [500,1000,1500,2000]

        }
        sweeps = sweep(parameters)
        self.sweeper = ParamSweeper(os.path.join(self.result_dir, "sweeps"), sweeps)
        logger.info('Number of parameters combinations %s', len(self.sweeper.get_remaining()))

    def workflow(self, comb):
        """ """
        comb_ok = False
        logger.info(slugify(comb) + \
                             ' starts to compile')
        try:
	    export = "source /opt/intel/bin/compilervars.sh intel64; "
	    
	    src_directory = "/home/arrouan/workspace/aevol/git/world/aevol/"
	    
	    bin_directory = "/home/arrouan/workspace/aevol/compiled_binary/"
    
	    configure_option = "--with-tracing --without-x"
	    
	    if comb['blas'] == 'openblas':
	      configure_option += " --with-blas"
	    elif comb['blas'] == 'mkl':
	      configure_option += " --with-mkl"
	    elif comb['blas'] == 'atlas':
	      configure_option += " --with-atlas"
	      
	    if comb['compilator'] == 'intel':
	      configure_option += " CXX=icc"
	      
	    full_bin_directory = bin_directory + comb['experiment']+'_'+comb['compilator']+'_'+comb['parallel']+'_'+comb['blas']
	    
	    try:
	      os.mkdir(full_bin_directory)
            except:
	      for f in os.listdir(full_bin_directory):
		os.remove(full_bin_directory + "/" + f)
		
	    p = Process(export+'cd '+src_directory+'; autoreconf; ./configure '+configure_option+'; make clean; make; cp src/aevol_run '+full_bin_directory+'/; cp src/aevol_create '+full_bin_directory+'/')
	    p.shell = True
	    #
	    p.run()
	    
	    print p.stdout
	    
            comb_ok = True
        finally:
            if comb_ok:
                self.sweeper.done(comb)
	        logger.info(slugify(comb) + \
                             ' has been done')
            else:
                self.sweeper.cancel(comb)
                logger.warning(slugify(comb) + \
                            ' has been canceled')
        logger.info(style.step('%s Remaining'),
                        len(self.sweeper.get_remaining()))

    def is_job_alive(self):
        rez=get_oar_job_info(self.oar_job_id)
        if (rez["start_date"]+rez["walltime"] > time.time()):
            return True
        else:
            return False

if __name__ == "__main__":
    engine = compil_aevol()
    engine.start()



#!/usr/bin/env python

import os
import numpy
import sys
import getopt
import png
import glob

def treat_generation(gfile,namepng,xsize,ysize,minv=0.,maxv=1.,colors=(0,255,0)):
    (rscale,gscale,bscale)=colors
    table=numpy.zeros((xsize,ysize))
    f = open(gfile,'r')
    for line in f:
        if len(line.split())==3:
            [x,y,z]=line.split()
            value=float(z)
            if value<minv:
                value=minv
            if value>maxv:
                value=maxv
            table[int(x),int(y)]=value
    f.close()
    pix=[tuple(reduce(lambda x,y:x+y, [(int((j-minv)/(maxv-minv)*rscale),int((j-minv)/(maxv-minv)*gscale),int((j-minv)/(maxv-minv)*bscale)) for j in i])) for i in table]

    pngfile=open(namepng,'wb')
    wr=png.Writer(xsize,ysize)
    wr.write(pngfile,pix)
    pngfile.close()
        
def printhelp():
    name=os.path.basename(sys.argv[0])
    print name +': a script to produce a movie from an aevol simulation with dumps activated.'
    print 'You need to have ffmpeg installed and in your path.'
    print 'Usage:'
    print '\t' + name + ' -i inputname [-s firstgen] [-e lastgen] [-p patchsize] [-m minvalue] [-M maxvalue] [-c color] [-o] [-k] sim1 sim2 .. simN'
    print 'or'
    print '\t' + name + ' -h'
    print 'Example:'
    print '\t' + " ./movies.py -i fitness_metabolic -p 40 -c '255 255 255' -m 0 -M 1 -s 30 -e 100 ~/myaevolsimulation"
    print 'See source code for a ful description of parameters and default values'
    exit(0)


opts, args = getopt.getopt(sys.argv[1:], "i:s:e:p:m:M:c:kho", ["inputname","start","end","patchsize","minvalue","maxvalue","color","keep","help","overwrite"])

start=None
end=None
inputname=None
patchsize=20
minvalue=0.
maxvalue=1.
colors=(0,255,0)
keepinter=False
overwrite=False

for o, a in opts:
    if o=='-i' or o=='--inputname': # Name of the output to treat (basename of the matching dump files, for example 'fitness_metabolic'). 
        inputname=a
    elif o=='-s' or o =='--start': # (optional) First generation to treat
        start=int(a)
    elif o=='-e' or o=='--end': # (optional) Last generation to treat
        end=int(a)
    elif o=='-p' or o=='--patchsize': # (optional) Size of each location in pixels (default 20)
        patchsize=int(a)
    elif o=='-m' or o=='--minvalue': # (optional) Value that will be scaled to 0 (black)
        minvalue=float(a)
    elif o=='-M' or o=='--maxvalue': # (optional) Value that will be scaled to 1 (full color specified by -c option)
        maxvalue=float(a)
    elif o=='-c' or o=='--color': # (optional) Color to give to maxvalue
        if (a=='red'):
            colors=(255,0,0)
        elif (a=='green'):
            colors=(0,255,0)
        elif (a=='blue'):
            colors=(0,0,255)
        elif len(a.split())==3:
            colors=map(lambda x: int(x), a.split())
        else:
            print 'Error: unrecognized argument given to -c option'
            exit(-1)
    elif o=='-k' or o=='--keep': # do not erase png files
        keepinter=True
    elif o=='-o' or o=='--overwrite': # re-generate png files if already existing
        overwrite=True
    elif o=='-h' or o=='--help':
        printhelp()
    else:
        print 'Error: unrecognized option ' + o
        exit(-1)

if not inputname:
    print 'Error: you must specifiy option -i (--inputname)'
    exit(-1)

smallestgen=1000000
biggestgen=None
warned=False

for arg in args: # If several folders are given as arguments treat all of them separatly
    first=True
    for filename in os.listdir(arg+'/stats/dump/'):
        if filename.startswith(inputname):
            ngen=int(filename.split('_').pop().split('.')[0])
            if ngen>biggestgen:
                biggestgen=ngen
            if ngen<smallestgen:
                smallestgen=ngen
            if first: # Use this first file to infer size of the grid
                mx=-1
                my=-1
                first=False
                temp=open(arg+'/stats/dump/'+inputname+'_%06d.out'%ngen)
                for line in temp:
                    if len(line)<=3 or line.startswith('#'):
                        continue
                    x=int(line.split()[0])
                    y=int(line.split()[0])
                    if x>mx:
                        mx=x
                    if y>my:
                        my=y
                temp.close()
                print "Detected x size: %d, detected y size: %d"%(mx+1,my+1)
            if (start and ngen<start) or (end and ngen>end): # do not treat generations that are not in given interval if any
                continue
            namepng='_%s%06d.png'%(inputname,ngen)
            if not os.path.exists(namepng) or overwrite:
                treat_generation(arg+'/stats/dump/'+inputname+'_%06d.out'%ngen,namepng,mx+1,my+1,minvalue,maxvalue,colors)
            elif not warned:
                print ''
                print "Warning: already existing png files found, we will use them instead of re-generating. If you want to re-generate them (to use different options), you first need to delete existing png files or to use option --overwrite"
                print ''
                warned=True
    if not start:
        start=smallestgen
    if not end:
        end=biggestgen
    os.system("ffmpeg -start_number " + str(start) + " -i _" + inputname + "%06d.png -vframes " + str(end-start) + " -r 6  -vcodec png -s "+str((mx+1)*patchsize)+"x"+str((my+1)*patchsize)+" -sws_flags neighbor -sws_dither none " + arg + '/' + inputname + ".mov")
    if not keepinter:
        os.system('rm _' + inputname + '*.png')




#! /usr/bin/python

import os
import shutil
import sys
import stat
import random
import tarfile

# ****************************** SETTINGS ******************************
exp_name            = 'my_exp'
nb_seeds            = 5
rearrangement_rates = ['1e-6', '2e-6', '5e-6', '1e-5', '2e-5', '5e-5', '1e-4']
mutation_rates      = ['1e-6', '2e-6', '5e-6', '1e-5', '2e-5', '5e-5', '1e-4']
nb_gener            = 50000

aevol_version       = 'aevol-3.1.0'
aevol_dir           = '/sps/liris/aevol/' + aevol_version
aevol               = aevol_dir + '/src/aevol_IN2P3'
backup_script       = aevol_dir + '/src/misc/make_big_backup_IN2P3.py'
exp_dir             = '/sps/liris/dparsons/sim/' + exp_name
# **********************************************************************

# Anti overwrite
if os.path.exists(exp_name):
  sys.exit( 'prepare_experiment: creating experiment \'' + exp_name + '\': Directory exists\n' )
if not os.path.exists( 'param.in' ):
  sys.exit( 'prepare_experiment: creating experiment \'' + exp_name + '\': No param.in file provided\n' )
  


# Create the experiment directory and its 'scripts' directory
os.makedirs( exp_name )
os.chdir( exp_name )
os.makedirs( 'scripts' )
submit_all_sh_file_name = 'scripts/submit_all.sh'
submit_all_sh_file      = open( submit_all_sh_file_name, 'w' )
aevol_version_info_file = open( 'aevol_version_info.txt', 'w' )
aevol_version_info_file.write( aevol_version + '\n' )
aevol_version_info_file.close()
nb_gener_info_file = open( 'nb_gener.txt', 'w' )
nb_gener_info_file.write( str(nb_gener) + '\n' )
nb_gener_info_file.close()


# Initialize random generator
random.seed()


# Create the directory, param file and script for each simulation
for rear_rate in rearrangement_rates:
  for mut_rate in mutation_rates:
    for i in range(1, nb_seeds+1):
      
      # **********************************************************************
      # 1) Set a few variables
      # **********************************************************************
      sim_dir           = 'rear_' + rear_rate + 'mut_' + mut_rate + '/seed' + str(i)
      sps_sim_dir       = exp_dir + '/' + sim_dir
      sim_name          = 'rear_' + rear_rate + 'mut_' + mut_rate + '_seed' + str(i)
      job_name          = sim_name
      
      
      # **********************************************************************
      # 2) Create simulation directory and parameter file (using template)
      # **********************************************************************
      os.makedirs( sim_dir )
      
      template_param_file = open('../param.in', 'r')
      new_param_file = open( sim_dir + '/param.in', 'w' )
      
      for cur_line in template_param_file:
        if cur_line.find('SEED') == 0:
          new_param_file.write('SEED ' + str(random.randint(1, 10000000)) + '\n')
        elif cur_line.find('POINT_MUTATION_RATE') == 0:
          new_param_file.write('POINT_MUTATION_RATE   ' + mut_rate + '\n')
        elif cur_line.find('SMALL_INSERTION_RATE') == 0:
          new_param_file.write('SMALL_INSERTION_RATE  ' + mut_rate + '\n')
        elif cur_line.find('SMALL_DELETION_RATE') == 0:
          new_param_file.write('SMALL_DELETION_RATE   ' + mut_rate + '\n')
        elif cur_line.find('DUPLICATION_RATE') == 0:
          new_param_file.write('DUPLICATION_RATE      ' + rear_rate + '\n')
        elif cur_line.find('DELETION_RATE') == 0:
          new_param_file.write('DELETION_RATE         ' + rear_rate + '\n')
        elif cur_line.find('TRANSLOCATION_RATE') == 0:
          new_param_file.write('TRANSLOCATION_RATE    ' + rear_rate + '\n')
        elif cur_line.find('INVERSION_RATE') == 0:
          new_param_file.write('INVERSION_RATE        ' + rear_rate + '\n')
        else:
          new_param_file.write(cur_line)
          
      new_param_file.close()
      template_param_file.close()
      
      
      # **********************************************************************
      # 3) Build the scripts that will be launched on the cluster
      # **********************************************************************
      script_file_name  = 'scripts/' + job_name + '.sh'
      script_file       = open(script_file_name, 'w')
      script_file.write('#!/bin/sh\n')
      script_file.write('#PBS -N ' + job_name + ' # Job name\n')
      script_file.write('#PBS -q T                             # Execution Class\n')
      script_file.write('#PBS -l platform=LINUX                # Execution Platform\n')
      script_file.write('#PBS -l u_sps_liris                   # We use the Semi Permanent Storage\n')
      script_file.write('#PBS -l T=4286000                     # Duration (in normalized units)\n')
      script_file.write('#PBS -l M=4096MB                      # Virtual Memory in MB\n')
      script_file.write('#PBS -l scratch=30GB                  # Scratch size in MB (disk space on calculator)\n')
      script_file.write('#PBS -l spool=500KB                   # Spool size in KB (space for stdout and stderr)\n')
      script_file.write('#PBS -l model=Xeon                    # Processor\n')
      script_file.write('#PBS -eo                              # Redirect stderr to stdout\n')
      script_file.write('#PBS -mb                              # Send Mail on Begin\n')
      script_file.write('#PBS -me                              # Send Mail on End\n')
      script_file.write('\n\n\n')
      script_file.write('# Simulation directories -> Where you have your input files and where you want your output files to be copied\n')
      script_file.write('SPS_BASE_DIR="' + exp_dir + '"\n')
      script_file.write('SIM_DIR="' + sim_dir + '"\n')
      script_file.write('SIM_NAME="' + sim_name + '"\n')
      script_file.write('SPS_SIM_DIR="' + sps_sim_dir + '"\n')
      script_file.write('\n')
      script_file.write('# Binary program path.\n')
      script_file.write('EXEC="' + aevol + '"\n')
      script_file.write('\n')
      script_file.write('# Create the simulation dir and cd into it\n')
      script_file.write('mkdir -p $SIM_DIR             # Create the simulation subdirectory on the calculator\n')
      script_file.write('cd $SIM_DIR                   # cd on the calculator\n')
      script_file.write('\n')
      script_file.write('# Copy the param.in file\n')
      script_file.write('cp $SPS_SIM_DIR"/param.in" .  # Copy param.in from SPS to the calculator\n')
      script_file.write('\n')
      script_file.write('# Write the sim_name.txt, SPS_dir.txt and cpu_info.txt files\n')
      script_file.write('echo ' + exp_name + ' > exp_name.txt\n')
      script_file.write('echo ' + sim_name + ' > sim_name.txt\n')
      script_file.write('echo ' + sps_sim_dir + ' > SPS_dir.txt\n')
      script_file.write('cp /proc/cpuinfo cpu_info.txt\n')
      script_file.write('\n')
      script_file.write('# Copy the big_backup script\n')
      script_file.write('cp ' + backup_script + ' make_big_backup.py\n')
      script_file.write('\n')
      script_file.write('# Run the program "$EXEC" for nb_gener generations\n')
      script_file.write('$EXEC -n ' + str(nb_gener) + ' > /dev/null\n')
      script_file.write('\n')
      script_file.write('# Make a .tar.gz from the result (with the directory structure $SIM_DIR)\n')
      script_file.write('cd ../..\n')
      script_file.write('tar zcf $SIM_NAME.tgz $SIM_DIR --remove-files\n')
      script_file.write('mv  $SIM_NAME.tgz $SPS_BASE_DIR/\n')
      script_file.close()
      
      submit_all_sh_file.write('qsub ' + exp_dir + '/' + script_file_name + '\n')
      os.chmod(script_file_name, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH)


submit_all_sh_file.close()
os.chmod(submit_all_sh_file_name, stat.S_IRWXU | stat.S_IRGRP | stat.S_IROTH)


# ***************************************************************
# 4) Put the freshly created files in a tarball (and delete them)
# ***************************************************************
os.chdir( '..' )
dirs_archive = tarfile.open( exp_name + '.tar.gz', 'w:gz')
dirs_archive.add( exp_name )
dirs_archive.close()


shutil.rmtree( exp_name )



from threading import Thread

def thread_main():
    print "hello"
    sleep 100000

if __name__ == "__main__":
    threads = {}
    for i in range(0,100):
        t = Thread(target=thread_main)
        threads[t] = {'host': i}
        t.daemon = True
        t.start()
        sleep(3)



#!/usr/local/bin/python

import os
import sys
import stat
import glob
import random
import tarfile
import shutil

#~ if len(sys.argv) != 2:
  #~ sys.stdout.write( 'ERROR : missing \'generation number\' operand\n' )
#~ else:
  #~ num_gener = sys.argv[1]
  
  
if 1 == 1:
  ################################################################################################################
  ## Retreive the simulation name, the number of the last saved generation and the SPS directory of the experiment
  ################################################################################################################
  # Get simulation name
  sim_name_file = open( 'sim_name.txt', 'r' )
  sim_name = (sim_name_file.readline()).rstrip( '\n' )
  sim_name_file.close()
  
  # Get number of the last saved generation
  last_gener_file = open( 'last_gener.txt', 'r' )
  last_gener = (last_gener_file.readline()).rstrip( '\n' )
  last_gener_file.close()
  
  # Get SPS dir
  sps_dir_file = open( 'SPS_dir.txt', 'r' )
  sps_dir = (sps_dir_file.readline()).rstrip( '\n' )
  sps_dir_file.close()
  
  
  
  ####################################################################################
  ## Open the general archive: a tgz file to put our "big backup"
  ####################################################################################
  general_archive_name = sim_name + '_' + last_gener + '.tgz'
  general_archive = tarfile.open( general_archive_name, 'w:gz' )
  
  
  
  ##################################################################################
  ## Save a copy of all the stat files in a targzipped "stat_<last_gener>" directory
  ##################################################################################
  # mkdir stat_<last_gener>
  os.makedirs( 'stat_' + last_gener )
  
  # cp stat_*.out stat_<last_gener>
  for filename in glob.glob( os.path.join( '.', 'stat_*.out' ) ):
    shutil.copy( filename, 'stat_' + last_gener + '/' + filename )
  
  # tar zcf stat_<last_gener>.tgz stat_<last_gener>
  stat_archive = tarfile.open( 'stat_' + last_gener + '.tgz', 'w:gz' )
  stat_archive.add( 'stat_' + last_gener )
  stat_archive.close()
  
  # rm -rf stat_<last_gener>
  shutil.rmtree( 'stat_' + last_gener )
  
  
  
  ####################################################################################
  ## Put stat_<last_gener>.tgz in the general archive
  ####################################################################################
  general_archive.add( 'stat_' + last_gener + '.tgz' )
  general_archive.add( 'backup' )
  general_archive.add( 'tree' )
  
  # add *.txt to general archive
  for filename in glob.glob( os.path.join( '.', '*.txt' ) ):
    general_archive.add( filename )
  general_archive.close()
  
  # rm (stat, backup, tree)_<last_gener>.tgz
  os.remove( 'stat_' + last_gener + '.tgz' )
  
  # rm -f backup/* (keeping only the last backup)
  os.rename( 'backup', 'backup.tmp' )
  os.makedirs( 'backup' )
  os.rename( 'backup.tmp/gen_' + '{0:0>6}'.format( last_gener ) + '.ae', 'backup/gen_' + '{0:0>6}'.format( last_gener ) + '.ae' )
  shutil.rmtree( 'backup.tmp' )
  # rm -f tree/*
  shutil.rmtree( 'tree' )
  os.makedirs( 'tree' )
  
  # move general archive to SPS
  shutil.copy( general_archive_name, sps_dir )
  os.remove( general_archive_name )
  


#!/usr/bin/env python
# coding: utf-8

import argparse
import pandas as pd
from pathlib import Path
import sys

def main():
  # Get command line arguments
  parser = argparse.ArgumentParser(description='Compare stats files')
  parser.add_argument(
      'fileA', type=Path,
      help='first file to compare')
  parser.add_argument(
      'fileB', type=Path,
      help='second file to compare')

  args = parser.parse_args()

  # Compare files
  compare(args.fileA, args.fileB)

  sys.exit(0)


def compare(fileA: Path, fileB: Path):
  dfA = pd.read_csv(fileA, comment='#', sep=' ')
  dfB = pd.read_csv(fileB, comment='#', sep=' ')

  # Assert that frames are equal
  # assert_frame_equal internally uses numpy.isclose with rtol and atol (relative and absolute tolerance)
  # parameters rtol and atol are explicitly set to their default values for future reference
  try:
    pd.testing.assert_frame_equal(dfA, dfB, rtol=1e-5, atol=1e-8)
  except AssertionError as e:
    print(e)
    sys.exit(1)


if __name__ == '__main__':
  main()
